import "../stylesheets/PlantDetails.css";
import "font-awesome/css/font-awesome.min.css";
import AgricultureIcon from '@mui/icons-material/Agriculture';
import HeartBrokenIcon from '@mui/icons-material/HeartBroken';
import {ReactNode, useEffect, useState} from "react";
import {Plant} from "../utils/schemas";
import {useParams} from "react-router";
import {IBackendConnector} from "../utils/IBackendConnector";
import useGrowbrosFetcher from "../utils/useGrowbrosFetcher";
import {FetchResult} from "../utils/FetchResult.ts";
import StatusMessage from "./StatusMessage.tsx";

function PlantDetails() {
    const {plantId} = useParams();

    const [plant, setPlant] = useState<Plant | null>(null);
    const [error, setError] = useState<object>();

    const bc: IBackendConnector = useGrowbrosFetcher();

    useEffect(() => {
        (async () => {
            const result = await bc.getSinglePlant(Number(plantId!));
            if (result.err) {
                setError(result.err);
            } else {
                setPlant(result.value!);
            }
        })();
    }, [setPlant, setError]);

    if (plant) {
        const attributes = [
            {title: "Lateinischer Name", value: plant.latinName},
            {title: "Beschreibung", value: plant.description},
            {title: "Herkunft", value: plant.origin},
            {title: "Anbautipps", value: plant.growingTips},
            {
                title: "Anbauzeitraum",
                value: `Von Kalenderwoche ${plant.plantWeekStart} bis Kalenderwoche ${plant.plantWeekEnd}`,
            },
            {
                title: "Erntezeitraum",
                value: `Von Kalenderwoche ${plant.harvestWeekStart} bis Kalenderwoche ${plant.harvestWeekEnd}`,
            },
            {title: "Bodentyp", value: plant.groundType},
            {title: "Nährstoffbedarf", value: plant.nutrientDemand},
            {title: "Wasserbedarf", value: plant.waterDemand},
            {title: "Lichtbedarf", value: plant.lightingDemand},
        ];

        return (
            <>
                {
                    error
                        ? <p>FEHLER!</p>
                        : <div style={{display: "flex", flexDirection: "column"}}>
                            <div className="plantDetails">
                                <h1>{plant.name}</h1>
                                <img src={plant.imageUrl}></img>

                                {attributes.map(
                                    (attribute, index) =>
                                        !!attribute.value && (
                                            <div className="infoBox" key={index}>
                                                <div className="title">{attribute.title}</div>
                                                <p>{attribute.value}</p>
                                            </div>
                                        )
                                )}
                            </div>
                            <div
                                className="buttons"
                                style={{
                                    display: "flex",
                                    justifyContent: "space-around",
                                    margin: "0 0 40px 0",
                                }}
                            >
                                <ActionButton buttonTitle={"Add to garden"}
                                              buttonContent={<i className="fa fa-leaf"></i>}
                                              buttonAction={() => bc.addToGarden(plant.id)}/>
                                <ActionButton buttonTitle={"Remove from garden"}
                                              buttonContent={<AgricultureIcon/>}
                                              buttonAction={() => bc.removeFromGarden(plant.id)}/>
                                <ActionButton buttonTitle={"Add to wishlist"}
                                              buttonContent={<i className="fa fa-heart"></i>}
                                              buttonAction={() => bc.addToWishlist(plant.id)}/>
                                <ActionButton buttonTitle={"Remove from wishlist"}
                                              buttonContent={<HeartBrokenIcon sx={{fontSize: 19}}/>}
                                              buttonAction={() => bc.removeFromWishlist(plant.id)}/>
                            </div>
                        </div>
                }
            </>
        );
    }
}

type ActionButtonProps = {
    buttonTitle: string,
    buttonContent: ReactNode,
    buttonAction: () => Promise<FetchResult<unknown>>;
}

function ActionButton({buttonContent, buttonTitle, buttonAction}: ActionButtonProps) {
    const [status, setStatus] = useState<"error" | "success">();
    const message = () => {
        if (status === "error") {
            return "Something went wrong..."
        } else {
            return "Success!"
        }
    };

    const statusVisibleFor = 3000;

    const handleButtonClick = async () => {
        const result = await buttonAction();
        if (result.err) {
            setStatus("error");
        } else {
            setStatus("success")
        }
        setTimeout(() => setStatus(undefined), statusVisibleFor);
    };

    return (
        <>
            <button title={buttonTitle} onClick={handleButtonClick}>
                {buttonContent}
            </button>
            {status && <StatusMessage message={message()} status={status} visibleFor={3000}/>}
        </>
    );
}

export default PlantDetails;
