import { satisfies } from "semver"
import z from "zod"

const groundType=z.enum(["HEAVY", "MEDIUM", "LIGHT"])
const waterDemand=z.enum(["DRY","WET","VERY_WET"])
const lightingDemand=z.enum(["LOW","MEDIUM","HIGH"])
const nutrientDemand=z.enum(["LOW","MEDIUM","HIGH"])
const weekNumber= z.number().int().min(0).max(51)

export type WaterDemand = z.infer<typeof waterDemand>;
export type GroundType= z.infer<typeof groundType>
export type LightingDemand= z.infer<typeof lightingDemand>
export type NutrientDemand= z.infer<typeof nutrientDemand>

export type NutrientDemandTranslated = "Niedrig" | "Mittel" | "Hoch";
export type WaterDemandTranslated = "Trocken" | "Feucht" | "Sehr feucht";
export type LightingDemandTranslated = "Niedrig" | "Mittel" | "Hoch";
export type GroundTypeTranslated = "Leicht" | "Mittel" | "Schwer";

export const PlantSchema = z.object({
    id: z.number(),
    name: z.string(),
    latinName: z.string().nullable(),
    description: z.string().nullable(),
    origin: z.string().nullable(),
    growingTips: z.string().nullable(),
    plantWeekStart: z.number().int().gte(0).nullable(),
    plantWeekEnd: z.number().int().gte(0).nullable(),
    harvestWeekStart: z.number().int().gte(0).nullable(),
    harvestWeekEnd: z.number().int().gte(0).nullable(),
    groundType: groundType.transform(s => translateGroundType(s)).nullable(),
    waterDemand: waterDemand.transform(s=>translateWaterDemand(s)).nullable(),
    lightingDemand: lightingDemand.transform(s=>translateLightingDemand(s)).nullable(),
    nutrientDemand: nutrientDemand.transform(s=>translateNutrientDemand(s)).nullable(),
    imageUrl: z.string().optional()
  });
  
  export type Plant = z.infer<typeof PlantSchema>;

  
  
function translateGroundType(key: GroundType) { 
    const map = {
        HEAVY: "Schwer",
        MEDIUM: "Mittel",
        LIGHT: "Leicht"
    } satisfies Record<GroundType, GroundTypeTranslated>
    return map[key];
  }

export function translateGroundTypeReverse(key: GroundTypeTranslated){
    const map = {
        Schwer: "HEAVY",
        Mittel: "MEDIUM",
        Leicht: "LIGHT"
    } satisfies Record<GroundTypeTranslated, GroundType>
    return map[key]
}

function translateLightingDemand(key: LightingDemand){
    const map = {
        LOW: "Niedrig",
        MEDIUM: "Mittel",
        HIGH: "Hoch"
    } satisfies Record<LightingDemand, LightingDemandTranslated>
    return map[key];
  }

 export function translateLightingDemandReverse(key: LightingDemandTranslated){
    const map = {
        Niedrig :"LOW",
        Mittel : "MEDIUM",
        Hoch: "HIGH"
    }satisfies Record<LightingDemandTranslated,LightingDemand>
    return map[key]
  }
  

  function translateWaterDemand(key: WaterDemand){
    const map = {
        DRY: "Trocken",
        WET: "Feucht",
        VERY_WET: "Sehr feucht"
    } satisfies Record<WaterDemand, WaterDemandTranslated> 
    return map[key];
  }

  export function translateWaterDemandReverse(key: WaterDemandTranslated){
    const map = {
        Trocken: "DRY",
        Feucht: "WET",
        "Sehr feucht" :"VERY_WET"
    } satisfies Record<WaterDemandTranslated, WaterDemand> 
    return map[key];
  }



function translateNutrientDemand(key: NutrientDemand){
    const map = {
        LOW: "Niedrig",
        MEDIUM: "Mittel",
        HIGH: "Hoch"
    } satisfies Record<NutrientDemand, NutrientDemandTranslated>
    return map[key];
  }

  export function translateNutrientDemandReverse(key: "Niedrig" | "Mittel" |"Hoch"){
    const map = {
        Niedrig :"LOW",
        Mittel : "MEDIUM",
        Hoch: "HIGH"
    }satisfies Record<NutrientDemandTranslated, NutrientDemand>
    return map[key]
  }

export const SearchRequestSchema = z.object({
    searchTerm: z.string().optional(),
    waterDemand: waterDemand.optional(),
    nutrientDemand: nutrientDemand.optional(),
    groundType: groundType.optional(),
    lightingDemand: lightingDemand.optional(),
    plantWeekStart: weekNumber.optional(),
    plantWeekEnd: weekNumber.optional(),
    growthDurationMin: weekNumber.optional(),
    growthDurationMax: weekNumber.optional()
  })

export type SearchRequest = z.infer<typeof SearchRequestSchema>;

function createCustomPageSchema<ItemType extends z.ZodTypeAny>(item: ItemType) {
    return z.object({
        currentPage: z.number().int(),
        pageSize: z.number().int(),
        content: z.array(item)
    });
}

export const CustomPlantPageSchema = createCustomPageSchema(PlantSchema);
export type CustomPlantPage = z.infer<typeof CustomPlantPageSchema>;
