package hdm.mi.growbros.service;

import hdm.mi.growbros.exceptions.PlantNotFoundException;
import hdm.mi.growbros.models.dto.CustomPageDto;
import hdm.mi.growbros.models.dto.DeletedPlantDto;
import hdm.mi.growbros.models.dto.PlantUpdateRequest;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.search.PlantSpecification;
import hdm.mi.growbros.models.search.SearchRequest;
import hdm.mi.growbros.repositories.GardenRepository;
import hdm.mi.growbros.repositories.PlantRepository;
import hdm.mi.growbros.repositories.WishListRepository;
import hdm.mi.growbros.util.EntityMapper;
import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@RequiredArgsConstructor
public class PlantsService {
    private final static Logger log = LoggerFactory.getLogger(PlantsService.class);
    private final static int MAX_PAGE_SIZE = 100;

    private final PlantRepository plantRepository;
    private final WishListRepository wishListRepository;
    private final GardenRepository gardenRepository;
    private final PlantSpecification plantSpecification;
    private final EntityMapper entityMapper;

    public CustomPageDto<Plant> getPlants(int pageNumber, int pageSize) {
        pageNumber = Math.max(0, pageNumber);
        pageSize = Math.min(MAX_PAGE_SIZE, pageSize);

        final Pageable page = PageRequest.of(pageNumber, pageSize);
        List<Plant> pageContent = plantRepository.findAll(page).getContent();
        return new CustomPageDto<>(pageNumber, pageContent.size(), pageContent);
    }

    public Collection<Plant> getRandomPlants(int numberOfPlants) {
        numberOfPlants= Math.min(MAX_PAGE_SIZE, numberOfPlants);
        long plantCount= plantRepository.count();
        int lastPage= (int)(plantCount/numberOfPlants)-1;
        int randomPage= (int) (Math.random()*lastPage);

        CustomPageDto<Plant> randomPlantPage= getPlants(randomPage, numberOfPlants);
        List<Plant> allPlantsOfPage = new ArrayList<>(randomPlantPage.content());
        Collections.shuffle(allPlantsOfPage);

        return allPlantsOfPage;
    }

    public Collection<Plant> getPlantsSearchedFor(SearchRequest searchRequest) {
        Specification<Plant> spec = plantSpecification.withSearchCriteria(searchRequest);
        return plantRepository.findAll(spec);
    }

    @Secured({"ADMIN"})
    public void createPlant(Plant plant) {
        log.info("Saving plant '{}' to repository.", plant.getName());
        plantRepository.save(plant);
    }

    public Optional<Plant> findById(long id) {
        return plantRepository.findById(id);
    }

    @Transactional
    @Secured({"ADMIN"})
    public DeletedPlantDto deletePlant(long id) {
        Optional<Plant> plant = plantRepository.findById(id);
        if (plant.isEmpty()) {
            return new DeletedPlantDto(null, 0, 0);
        }
        log.info("Deleting plant with id {}.", id);
        int deletedGardenEntries = gardenRepository.deleteAllByPlant_Id(id);
        int deletedWishlistEntries = wishListRepository.deleteAllByPlant_Id(id);
        plantRepository.deleteById(id);
        log.info("Plant '{}' was deleted. {} garden entries and {} wishlist entries were affected", plant.get().getName(), deletedGardenEntries, deletedWishlistEntries);
        return new DeletedPlantDto(plant.get(), deletedGardenEntries, deletedWishlistEntries);
    }

    @Transactional
    @Secured({"ADMIN"})
    public Plant updatePlant(long id, PlantUpdateRequest updateRequest) {
        Plant plant = plantRepository.findById(id).orElseThrow(() -> new PlantNotFoundException(id));
        entityMapper.map(updateRequest, plant);
        return plantRepository.save(plant);
    }
}
