package hdm.mi.growbros.service;

import hdm.mi.growbros.exceptions.PlantNotFoundException;
import hdm.mi.growbros.models.WishListEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.repositories.PlantRepository;
import hdm.mi.growbros.repositories.WishListRepository;
import hdm.mi.growbros.util.DateTimeUtils;
import jakarta.transaction.Transactional;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;


@Service
@RequiredArgsConstructor
public class WishListService {

    private final WishListRepository wishListRepository;
    private final PlantRepository plantRepository;

    public List<Plant> getWishedPlants(User user, String sort) {
        if (sort == null) return getAllPlantsOrderedByName(user);
        return switch (sort) {
            case "createdAt" -> getAllPlantsOrderedByCreatedDate(user);
            case "plantDate" -> getAllPlantsOrderedByNextPossiblePlantDate(user);
            case "currentPlantable" -> getCurrentPlantablePlants(user);
            default -> getAllPlantsOrderedByName(user);
        };
    }

    public void addPlantToWishList(Long plantId, User user) {
        Plant plant = plantRepository.findById(plantId).orElseThrow(() -> new PlantNotFoundException(plantId));

        if (wishListRepository.existsByUserAndPlant(user, plant)) {
            return;
        }

        WishListEntry wishListEntry = new WishListEntry();
        wishListEntry.setPlant(plant);
        wishListEntry.setUser(user);
        wishListRepository.save(wishListEntry);
    }

    @Transactional
    public void removeFromWishList(Long plantId, User user) {
        Long entryId = wishListRepository.findEntryIdByUserAndPlantId(user, plantId);
        wishListRepository.deleteByIdAndUser(entryId, user);
    }


    @Transactional
    public int clearWishList(User user) {
        return wishListRepository.deleteAllByUser(user);
    }

    private List<Plant> getAllPlantsOrderedByName(User user) {
        return mapWishListEntriesToPlants(
                wishListRepository.findByUser(user, Sort.by("plant.name"))
        );
    }

    private List<Plant> getAllPlantsOrderedByCreatedDate(User user) {
        return mapWishListEntriesToPlants(
                wishListRepository.findByUser(user, Sort.by("createdAt").descending())
        );
    }

    private List<Plant> getAllPlantsOrderedByNextPossiblePlantDate(User user) {
        return mapWishListEntriesToPlants(
                wishListRepository.findAllByNearestPlantingWeek(user, DateTimeUtils.getCurrentWeekForToday())
        );
    }

    private List<Plant> getCurrentPlantablePlants(User user) {
        return mapWishListEntriesToPlants(
                wishListRepository.findByCurrentPlantingWeek(user, DateTimeUtils.getCurrentWeekForToday())
        );
    }

    private List<Plant> mapWishListEntriesToPlants(Collection<WishListEntry> entries) {
        return entries
                .stream()
                .map(WishListEntry::getPlant)
                .toList();
    }

    public int countByUser(User user) {
        return wishListRepository.countByUser(user);
    }
}