package hdm.mi.growbros.service;

import hdm.mi.growbros.config.UserTestData;
import hdm.mi.growbros.exceptions.PlantNotFoundException;
import hdm.mi.growbros.models.GardenEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.repositories.GardenRepository;
import hdm.mi.growbros.repositories.PlantRepository;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Sort;
import org.springframework.test.context.ActiveProfiles;

import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.when;

@SpringBootTest
@ActiveProfiles("test")
class GardenServiceTest {
    @MockBean
    private GardenRepository gardenRepository;
    @MockBean
    private PlantRepository plantRepository;

    @Autowired
    private GardenService gardenService;

    @Test
    void shouldThrow_whenAddingPlant_thatDoesNotExist() {
        //setup
        when(plantRepository.findById(100L)).thenReturn(Optional.empty());

        assertThrows(PlantNotFoundException.class, () -> gardenService.addPlantToGarden(100L, UserTestData.getUser1()));
    }

    @Test
    void shouldReturn_whenAddingExistingPlant() {
        //setup
        Plant plant = Plant.builder().name("Plant!").build();
        when(plantRepository.findById(1L)).thenReturn(Optional.of(plant));
        when(gardenRepository.save(
                any(GardenEntry.class))
        ).thenReturn(GardenEntry.builder().plant(plant).build());

        //assert
        var result = gardenService.addPlantToGarden(1L, UserTestData.getUser1());
        assertEquals(result, plant);
    }

    @Test
    void shouldReturnEntries_inCorrectOrder() {
        //setup
        Plant p1 = Plant.builder().name("AAA").build();
        Plant p2 = Plant.builder().name("BBB").build();
        Plant p3 = Plant.builder().name("CCC").build();
        var orderedByNameMock = List.of(p1, p2, p3);
        var orderedByCreatedMock = List.of(p2, p1, p3);
        var orderedByHarvestMock = List.of(p1, p3, p2);
        var orderedByPlantDateMock = List.of(p3, p2, p1);

        User user = UserTestData.getUser1();

        when(gardenRepository.findByUser(user, Sort.by("createdAt").descending())).thenReturn(mapPlantsToGardenEntries(orderedByCreatedMock, user));
        when(gardenRepository.findByUser(user, Sort.by("plant.name"))).thenReturn(mapPlantsToGardenEntries(orderedByNameMock, user));
        when(gardenRepository.findAllByNearestHarvestWeek(eq(user), any(int.class))).thenReturn(mapPlantsToGardenEntries(orderedByHarvestMock, user));
        when(gardenRepository.findAllByNearestPlantingWeek(eq(user), any(int.class))).thenReturn(mapPlantsToGardenEntries(orderedByPlantDateMock, user));

        assertAll(
                () -> assertEquals(orderedByNameMock, gardenService.getUserPlants(user, null)),
                () -> assertEquals(orderedByNameMock, gardenService.getUserPlants(user, "random stuff")),
                () -> assertEquals(orderedByCreatedMock, gardenService.getUserPlants(user, "createdAt")),
                () -> assertEquals(orderedByPlantDateMock, gardenService.getUserPlants(user, "plantDate")),
                () -> assertEquals(orderedByHarvestMock, gardenService.getUserPlants(user, "harvestDate"))
        );
    }

    private List<GardenEntry> mapPlantsToGardenEntries(List<Plant> plants, User user) {
        return plants.stream()
                .map(plant -> GardenEntry.builder().plant(plant).id(0L).user(user).build())
                .toList();
    }
}
